# 作成日 2024年1月12日-------------------------------------------------------------------------
# R version 4.2.2 (2022-10-31 ucrt)
# RStudio 2022.12.0

# 操作方法の練習
c("RStudioでコードを実行したい場合は、コードが書かれた行にカーソルを合わせて「ctrl」+「enter」")

c("複数行のコードを実行したければ、該当の行を選択して「ctrl」+「enter」")
3+7
2*2
2~10
inu <- 10
neko <- 20
inu+neko

# c("半角「#」のマークは、コードではなく、コメントとして認識される")



# -------------------------------------------------------------------------
# Chapter02を進めるにあたり、はじめての場合は以下のパッケージをインストールしてください。
# パッケージのインストール
pkg <- c("tidyverse", "ggplot2", "openxlsx", "patchwork", "cmprsk", "survminer", "ggsurvfit", "finalfit", "survival")
install.packages(pkg)


#　パッケージの呼び出し
library(tidyverse)
library(ggplot2)
library(openxlsx)
library(patchwork)
library(cmprsk)
library(survminer)
library(ggsurvfit)
library(finalfit)
library(survival)


# -------------------------------------------------------------------------
# Chapter02 疫学指標　発展学習
# 演習・実習2-1 疫学指標の基本
# 保健統計の指標（罹患率、有病率（有病割合））を算出できる。
# 10名の仮想データ（追跡10年）を用いた疫学指標の算出

# データ生成
id <- as.numeric(rep(1:10))
status <- as.numeric(c(1,0,0,0,0,1,0,1,1,0))
start_time <- as.numeric(c(0,0,1,5,2,2,0,0,1,0))
end_time <- as.numeric(c(7,10,10,10,10,5,10,8,4,10))
incidence_time <- as.numeric(c(3,1,0,0,9,4,0,3,2,2))
recovery_time  <- as.numeric(c(0,2,0,0,10,0,0,0,0,5))
censor         <- as.numeric(c(0,9,0,0,0,0,0,0,0,8))

sample_df <- data.frame(id, status, start_time, end_time, 
                        incidence_time, recovery_time, start_time, censor)

sample_df$status <- factor(sample_df$status, 
                           levels = c(0,1), 
                           labels = c("生存","死亡"))

glimpse(sample_df)
write.csv(sample_df, "data_sample.csv")


# 追跡状況の可視化
fig_cohort <- ggplot(sample_df) +
  #追跡期間の線を引く
  annotate("segment", x = 4.5, y = 1, xend = 4.5, yend = 10.5, 
           linetype = 1, color = "grey", alpha = 0.7, arrow = arrow(length = unit(0.5, "cm")))+
  geom_segment(data=subset(sample_df, incidence_time==0&censor==0),
               aes(x = start_time, xend = end_time, y = id, yend = id), 
               color = "grey", linewidth=1.8) +
  geom_segment(data=subset(sample_df, incidence_time>=1&recovery_time==0),
               aes(x = incidence_time, xend = end_time, y = id, yend = id),
               color = "grey", linewidth=1,  linetype="dashed") +
  geom_segment(data=subset(sample_df, incidence_time>=1),
               aes(x = start_time, xend = incidence_time, y = id, yend = id), 
               color = "grey", linewidth=1.8) +
  geom_segment(data=subset(sample_df, incidence_time>=1&end_time==10&censor==0),
               aes(x = recovery_time, xend = end_time, y = id, yend = id), 
               color = "grey", linewidth=1.8) +
  geom_segment(data=subset(sample_df, incidence_time>=1&end_time==10&censor!=0),
               aes(x = recovery_time, xend = censor, y = id, yend = id), 
               color = "grey", linewidth=1.8) +
  geom_segment(data=subset(sample_df, incidence_time>=1&end_time==10),
               aes(x = incidence_time, xend = recovery_time, y = id, yend = id), 
               color = "grey", linewidth=1,  linetype="dashed") +
  #疾患の発生、死亡、打ち切り、追跡終了をプロット
  geom_point(data = subset(sample_df, censor==0),
             aes(x = end_time, y = id, shape = status), size = 5, stroke = 2)+
  geom_point(data=subset(sample_df, censor>=1),
             aes(x = censor, y = id), shape=24, size = 3, stroke = 1)+
  geom_point(data = subset(sample_df, incidence_time>=1),
             aes(x = incidence_time, y = id), shape=18, size = 5, stroke = 1)+
  #図のデザインを微調整
  geom_text(aes(x=1.5, y=2.5, label="罹病期間"), size=5)+
  geom_text(aes(x=4.5, y=10.7, label="時点(t)"), size=5, fontface = "bold")+
  geom_text(aes(x=7,  y=1.5, label="死亡"), size = 5, fontface = "bold")+
  geom_text(aes(x=8.8,  y=10, label="打ち切り"), size = 5)+
  geom_text(aes(x=9.8,  y=2, label="打ち切り"), size = 5)+
  geom_text(aes(x=10, y=7.5, label="生存"), size = 5, fontface = "bold")+
  scale_shape_manual(values = c(19, 4), 
                     name = "Status", 
                     labels = c("生存", "死亡")) +
  scale_x_continuous(breaks = seq(0, 10, by = 1), 
                     labels = seq(0, 10, by = 1),
                     limits = c(0,10.5), expand=c(0,0))+
  scale_y_reverse(breaks = seq(0, 10, by = 1), 
                  labels = seq(0, 10, by = 1),
                  limits = c(10.7, 1))+
  theme_classic()+
  theme(text = element_text(size = 10, face="bold"),
        plot.title = element_text(size=25),
        legend.position="right",
        axis.title.x = element_text(face="bold", size = 25),
        axis.title.y = element_text(face="bold", size = 25),
        axis.text.x = element_text(size = 20),
        axis.text.y = element_text(size = 20),
        axis.ticks.length.x = unit(4, "pt"),
        axis.ticks.length.y = unit(4, "pt"),
        legend.text = element_text(size = 20),
        plot.margin = unit(c(2,2,2,2), "lines"))+
  labs(title = "", 
       x = "フォローアップ期間 (年)",
       y = "ケース番号",
       color="Status")

# 右下のPlotペインに図を表示
fig_cohort


# 追跡状況に関する図の保存（Figure1）
ggsave(filename = "Fig1.jpeg", 
       plot = fig_cohort, 
       device = "jpeg", 
       width = 10, 
       height = 8, 
       unit = "in", 
       dpi = 600)


# 2) 罹患率の算出-------------------------------------------------------------------------
# 人時法（人年法）を用いた罹患率の算出
sample_df <- sample_df %>% 
  mutate(case = if_else(incidence_time >= 1, 1,0), #　疾患発生があれば１、そうでなければ0
         pyear = if_else(incidence_time > 0, 
                         incidence_time-start_time, 
                         end_time-start_time)) %>% # 追跡期間を算出
  arrange(id) # idで並び替え

glimpse(sample_df) # データフレームの確認
IR <- sample_df %>% summarise(IR = sum(case)/sum(pyear)) # 罹患率0.163（時^-1）
IR

# 3) 有病率（有病割合）の算出-------------------------------------------------------------------------
p.prop <- sample_df %>% 
  mutate(time_point = 4.5,
         case.exist = 
           case_when(incidence_time==0 ~ 0,
                     recovery_time==0 & time_point<=end_time & time_point>=incidence_time ~ 1,
                     time_point<=recovery_time & time_point>=incidence_time ~ 1,
                     TRUE ~0),
         at.risk =
           case_when(start_time>time_point|end_time<time_point ~ 0,
                     TRUE ~ 1))

prev.result <- p.prop %>% summarise(prevalence = sum(case.exist)/sum(at.risk)) # 有病率　0.5
prev.result


# カプラン-マイヤー法による累積罹患率の算出-------------------------------------------------------------------------
# ❚疾患の新規発生の場合
glimpse(sample_df)

sample_df <- sample_df %>% 
  mutate(case = if_else(incidence_time >= 1, 1,0), #　疾患発生があれば１、そうでなければ0
         pyear = if_else(incidence_time > 0, 
                         incidence_time-start_time, 
                         end_time-start_time)) %>% # 追跡期間を算出
  arrange(pyear) # 追跡期間で並び替え

#　pyearの重複を削除
df_km <- unique(sample_df[, c("pyear")]) 

#　caseの変数をpyearごとに合計
df_km <- as.data.frame(aggregate(case ~ pyear,data = sample_df, sum))

#　number at risk, survival fuctionなどを算出
df_km <- df_km %>%  
  mutate(number = if_else(case == 0, 1, case),
         cum_number = cumsum(number), # 人数の累積和
         at_risk = case_when(pyear == 1 ~ sum(number), # ある追跡年数時点においてリスクのある人数
                             pyear >= 2 ~ sum(number)-lag(cum_number)),
         q_death = case/at_risk,              # 疾患の発生確率
         p_alive = (at_risk-case)/(at_risk),  # 疾患の発生しない確率（or生存の確率）
         surv_function_S = cumprod(p_alive))  # 累積生存確率 cumulative probability of survival

#　結果の表示
df_km

#　10年間の累積罹患率
df_km %>% filter(pyear==10) %>% summarise(c.incidence=1-surv_function_S)


# 上記の結果についてカプラン-マイヤー曲線を表示
#累積生存割合として表示（この場合では疾患に罹患していない者の割合）
result <- survfit(Surv(pyear,case) ~ 1,
                  data=sample_df,
                  type="kaplan-meier")
summary(result)
plot(result, conf.int=F, mark.t = TRUE)


#累積罹患割合として表示
plot(result, fun="event", conf.int=F, mark.t = TRUE)


# カプラン-マイヤー曲線を「ggsurvplot」を用いて表示
# 累積罹患割合にする場合には、fun="event"を追加
ggsurvplot(result, 
           conf.int = T,
           palette = "Lancet",
           risk.table = T)



# Chapter02 疫学指標-------------------------------------------------------------------------
# 演習・実習2-2 関連の疫学指標
# 保健統計の関連の疫学指標を算出できる。
# 仮想の個票データを用いた関連の疫学指標の算出
# 相対危険（RR）、寄与危険（AR）、寄与危険割合（PAR）、集団寄与危険割合（PAF）、オッズ比（OR）

# 仮想データを読み込む
cohort_df <- read.xlsx("File2-6.xlsx", sheet = 1)
glimpse(cohort_df)
# id: 個票データの識別番号
# start_time: 追跡開始年月日（yyyymmdd）
# end_time: 追跡終了年月日（yyyymmdd）
# status: 追跡終了時の状態（alive, cvd, non-informative censoring）
# Bskipping: 朝食欠食状況（0, 欠食なし; 1, 欠食あり）
# cvd: 循環器疾患の発生（0, なし; 1, あり）
# age: 年齢（歳）
# sex: 性別（0, 男性; 1, 女性）
# confounder: 交絡変数（0, 交絡の要因なし; 1, 交絡の要因あり）
# ここでは交絡について特定の変数を具体化していない。そのため、わかりにくければ喫煙習慣（0, なし; 1,あり）などをイメージするとよい。

# person_year: 追跡年数（年）の算出
cohort_df <- cohort_df %>% mutate(person_year= (end_time-start_time)/365.25)
glimpse(cohort_df)


# 仮説を設定: 朝食欠食に曝露されているグループは、cvdの罹患リスクが高い
# caseとpyearの集計
total <- cohort_df %>% 
  group_by(Bskipping) %>% 
  summarise(cvd = sum(cvd), pyear=sum(person_year))
total

case_e0 <- total[1,2] # 曝露なしグループのcvdの合計
case_e1 <- total[2,2] # 曝露ありグループのcvdの合計

pyear_e0 <- total[1,3] # 曝露なしグループのpyearの合計
pyear_e1 <- total[2,3] # 曝露ありグループのpyearの合計

# 曝露なしグループの罹患率（/年）
ir_e0 <- case_e0/pyear_e0
ir_e0

# 曝露ありグループの罹患率（/年）
ir_e1 <- case_e1/pyear_e1
ir_e1

# 2) 前向きコホート研究： 相対危険(RR)の算出
RR <- ir_e1/ir_e0
RR

# 2) 前向きコホート研究： 寄与危険(AR)の算出
AR <- ir_e1-ir_e0
AR

# 2) 前向きコホート研究： 寄与危険割合(PAR)の算出
PAR <- (ir_e1-ir_e0)/ir_e1*100
PAR

# PAR を RRから算出
PAR_from_RR <- (RR-1)/RR*100
PAR_from_RR


# 3) 前向きコホート研究： 集団寄与危険割合(PAF)の算出
# cohort_dfにおける曝露の分布が母集団の分布を代表すると仮定した場合
exposure <- addmargins(table(cohort_df$Bskipping))
exposure
e0 <- exposure[1]/exposure[3] # 曝露なしの割合
e1 <- exposure[2]/exposure[3] # 曝露ありの割合


# 集団全体における罹患率（集団における曝露の分布で重み付け）
population.ir <- ir_e1*e1+ir_e0*e0
population.ir


#　集団寄与危険度
diff_p.ir <- population.ir-ir_e0
diff_p.ir


#　集団寄与危険割合(PAF)
PAF <- diff_p.ir/population.ir*100
PAF


#　RRからPAFの算出
PAF_from_RR <- (e1*(RR-1))/(e1*(RR-1)+1)*100
PAF_from_RR


#　結果をまとめて出力する
number <- rep(1:8) 
value <- c(RR,
           AR,
           PAR,
           PAR_from_RR,
           population.ir,
           diff_p.ir,
           PAF,
           PAF_from_RR)
index <- c("相対危険 (RR)",
           "寄与危険 (AR)",
           "寄与危険割合 (PAR)",
           "PAR from RR",
           "集団全体の罹患率",
           "集団寄与危険度",
           "集団寄与危険割合 (PAF）",
           "PAF from RR")
output <- cbind(number, index, value)
index <- as.data.frame(output)

#　ファイル保存
write.xlsx(index, file="result_index.xlsx")




#　5) 症例対照研究： オッズ比の算出-------------------------------------------------------------------------
df <- read.xlsx("File2-7.xlsx", sheet = 1)
glimpse(df)


# factor型の変数の作成
df$obesity_lab <- factor(df$obesity, levels = c(0,1), 
                         labels = c("non-obesity","obesity"))
df$cvd.event_lab <- factor(df$cvd.event, levels = c(0,1),
                           labels = c("no cvd","cvd"))
glimpse(df)


# 症例対照研究における曝露のオッズ比の算出
addmargins(table(df$obesity_lab, df$cvd.event_lab))
odds.ratio <- (60/40)/(150/250)

# cvdなしの肥満の曝露オッズに対するcvdありの肥満の曝露オッズの比
odds.ratio


#　上記の補足 -------------------------------------------------------------------------
#　ロジスティック回帰分析を用いた推定
model1 <- glm(cvd.event ~ obesity, 
              family = binomial(link = "logit"), 
              data = df)
summary(model1)

estimate <- (summary(model1)$coefficients[2,"Estimate"]) %>% 
  exp() %>% round(3)
ci.low <- (summary(model1)$coefficients[2,"Estimate"] +
             qnorm(0.05/2)*summary(model1)$coefficients[2,"Std. Error"]) %>% 
  exp() %>% round(3)
ci.up <- (summary(model1)$coefficients[2,"Estimate"] +
            qnorm(1-0.05/2)*summary(model1)$coefficients[2,"Std. Error"]) %>% 
  exp() %>% round(3)
paste("肥満のオッズ比：点推定値 (95%信頼区間下限, 95%信頼区間上限)=",
      estimate, " (", ci.low,",", ci.up, ")", sep="")

result <- data.frame(estimate=estimate, low=ci.low, high=ci.up)
result


# finalfitを用いたOdds ratioの可視化
explanatory = "obesity_lab"
dependent = 'cvd.event_lab'
df %>%
  or_plot(dependent, explanatory)


# 演習実習2-1，2-2 終了 ----------------------------------------------------------------------



# -------------------------------------------------------------------------
# Chapter02　コラム 比例ハザードモデルによるハザード比
# 多変量解析（Cox比例ハザードモデルによるハザード比と95%信頼区間の算出、可視化）
glimpse(cohort_df)

# 単変量モデル
cox.model_1 <- coxph(Surv(person_year, cvd)~Bskipping, data=cohort_df)
summary(cox.model_1)

# 多変量モデル
cox.model_2 <- coxph(Surv(person_year, cvd)~Bskipping+age+sex+confounder, data=cohort_df)
summary(cox.model_2)

#　上記のHR（95%CI）を可視化方法
# figureの作成
ggforest(cox.model_2)

# Adjusted cox model
fig.adjusted_cox <- coxph(Surv(person_year, cvd)~age+sex+confounder+strata(Bskipping), data=cohort_df) %>%
  survfit2() %>%
  ggsurvfit(type = "risk") +
  scale_color_manual(values = c('#FFDE59', '#00946B')) +
  scale_fill_manual(values = c('#FFDE59', '#00946B')) +
  labs(x = "Years", y = "Overall survival probability")+
  add_confidence_interval() +
  add_censor_mark()+
  add_risktable() +
  add_quantile()+
  scale_x_continuous(limits = c(0, 20),  breaks = seq(0, 20, by = 4))+
  scale_y_continuous(limits = c(0, 1))

ggsave(filename = "Fig.adj_cox.jpeg", 
       plot = fig.adjusted_cox, 
       device = "jpeg", 
       width = 10, 
       height = 8, 
       unit = "in", 
       dpi = 600)
